unit UTesteSOM;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Grids, Db, DBTables, DBGrids, DBCtrls, WaveHandle, FFT,
  StdCtrls, Buttons, ComCtrls, Som;

const
   DATASIZE = 1024;

type
  TFTesteSOM = class(TForm)
    FFT: TFFT;
    WaveHandle: TWaveHandle;
    Panel1: TPanel;
    Label2: TLabel;
    EdWaveFile: TEdit;
    BtnTest: TBitBtn;
    OpenDialog: TOpenDialog;
    StatusBar: TStatusBar;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    ImageWave: TImage;
    ImageFFT: TImage;
    Label1: TLabel;
    StaticText1: TStaticText;
    LbLength: TLabel;
    Label3: TLabel;
    LbLengthFormat: TLabel;
    Label5: TLabel;
    LbFormatTag: TLabel;
    Label6: TLabel;
    LbChannels: TLabel;
    Label7: TLabel;
    LbSamplesPerSec: TLabel;
    Label8: TLabel;
    LbAvgBytesPerSec: TLabel;
    Label9: TLabel;
    LbBlockAlign: TLabel;
    Label4: TLabel;
    LbDataLength: TLabel;
    BtnPlay: TBitBtn;
    SOMGer: TSOM;
    Panel6: TPanel;
    MemoTest: TMemo;
    Panel7: TPanel;
    LabelDigit: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
    procedure EdWaveFileDblClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure BtnTestClick(Sender: TObject);
    procedure BtnPlayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

    { Faixa de valores possveis }
    Minimo, Maximo: array[0..30] of extended;

    { Vetor para teste da rede SOM }
    X: array of extended;

    { Retngulo temporrio }
    ARect: TRect;

    { Tamanho da janela de visualizao da amostra }
    SamplerView: integer;

    { Volume }
    Volume: integer;

    { Parmetros de visualizao }
    { Altura, largura, diferena, quantidade de bin }
    BinHeight, BinWidth, DeltaBin, BinCount: integer;

    { Limite de corte de rudo de fundo }
    NoiseCut: integer;

    { Limite inferior de bins para corte }
    LowCut: integer;

    { Varivel temporrias para dados de FFT }
    DataFFT: array[1..DATASIZE*2] of double;

    { Varivel para armazenar espectro de freqncias }
    Espectro : array[1..DATASIZE] of double;

    { Mostra a estrutura do arquivo RIFF }
    procedure ShowStructure;

    { Mostra o sinal carregado do arquivo WAV }
    procedure ShowSignal;

    { Mostra a FFT esttica do sinal }
    procedure ShowFFT;

    { Mostra a FFT com janela deslizante }
    procedure ShowFFTWin;

  end;

var
  FTesteSOM: TFTesteSOM;

implementation

{$R *.DFM}

uses USoundRec;

procedure TFTesteSOM.FormActivate(Sender: TObject);
begin
   { Limpa campos }
   LbLength.Caption := '';
   LbLengthFormat.Caption := '';
   LbFormatTag.Caption := '';
   LbChannels.Caption := '';
   LbSamplesPerSec.Caption := '';
   LbAvgBytesPerSec.Caption := '';
   LbBlockAlign.Caption := '';
   LbDataLength.Caption := '';

   { Parmetros de vizualizao }
   BinWidth := 8;
   BinHeight := 12;
   DeltaBin := 3;
   BinCount := 42;
   NoiseCut := 10;
   LowCut := 12;
end;

procedure TFTesteSOM.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   Action := caFree;
   FTesteSOM := nil;
end;

procedure TFTesteSOM.FormResize(Sender: TObject);
begin
   { Divide a janela de visualizao em duas: Amostra e FFT }
   Panel5.Height := Panel3.Height div 2;
   Panel5.Repaint;
end;

procedure TFTesteSOM.EdWaveFileDblClick(Sender: TObject);
begin
   if OpenDialog.Execute then
   begin
      { Abre o arquivo e analisa no WaveHandle }
      BtnTest.Enabled := True;
      BtnPlay.Enabled := True;
      EdWaveFile.Text := ExtractFileName(OpenDialog.FileName);
      OpenDialog.InitialDir := ExtractFilePath(OpenDialog.FileName);
      WaveHandle.Clear;
      WaveHandle.WAVFile := EdWaveFile.Text;
      WaveHandle.Load;

      ARect := Rect(0, 0, ImageWave.Width, ImageWave.Height);
      ImageWave.Canvas.Brush.Color := clBlack;
      ImageWave.Canvas.FillRect(ARect);
      ShowStructure;
      ShowSignal;
      ShowFFT;
   end;
end;

procedure TFTesteSOM.ShowStructure;
begin
   LbLength.Caption := Format('%d',[WaveHandle.Length]);
   LbLengthFormat.Caption := Format('%d',[WaveHandle.LengthFormat]);
   LbFormatTag.Caption := Format('%d',[WaveHandle.FormatTag]);
   LbChannels.Caption := Format('%d',[WaveHandle.Channels]);
   LbSamplesPerSec.Caption := Format('%d',[WaveHandle.SamplesPerSec]);
   LbAvgBytesPerSec.Caption := Format('%d',[WaveHandle.AvgBytesPerSec]);
   LbBlockAlign.Caption := Format('%d',[WaveHandle.BlockAlign]);
   LbDataLength.Caption := Format('%d',[WaveHandle.LengthData]);

   SamplerView := WaveHandle.LengthData div 16384;
   if SamplerView = 0 then SamplerView := 1;
end;

procedure TFTesteSOM.ShowSignal;
var i, j, n : integer;
   k : longint;
   Volume : longint;
   ARect : TRect;
begin
   ARect := Rect(0, 0, ImageWave.Width, ImageWave.Height);
   ImageWave.Canvas.Brush.Color := clBlack;
   ImageWave.Canvas.FillRect(ARect);

   Volume := 65536 * 1; //SpinEdit1.Value;

   ImageWave.Canvas.Pen.Color := clLime;
   if WaveHandle.Channels = 1 then
   begin
      k := 0;
      for j := 0 to WaveHandle.NumBuffers-1 do
      begin
         i := 0;
         while i < 32768 do
         begin
            ImageWave.Canvas.MoveTo(Round(k * 2 * ImageWave.Width / WaveHandle.LengthData),
                                    (ImageWave.Height div 2) - WaveHandle.Buffer[j][i]*ImageWave.Height div Volume);
            Inc(k, SamplerView);
            Inc(i, SamplerView);
            ImageWave.Canvas.LineTo(Round(k * 2 * ImageWave.Width / WaveHandle.LengthData),
                                    (ImageWave.Height div 2) - WaveHandle.Buffer[j][i]*ImageWave.Height div Volume);
         end;
      end;

      // resto
      i := 0;
      n := WaveHandle.NumBuffers;
      while i < WaveHandle.BytesRemain do
      begin
         ImageWave.Canvas.MoveTo(Round(k * 2 * ImageWave.Width / WaveHandle.LengthData),
                                 (ImageWave.Height div 2) - WaveHandle.Buffer[n][i]*ImageWave.Height div Volume);
         Inc(k, SamplerView);
         Inc(i, SamplerView);
         ImageWave.Canvas.LineTo(Round(k * 2 * ImageWave.Width / WaveHandle.LengthData),
                                 (ImageWave.Height div 2) - WaveHandle.Buffer[n][i]*ImageWave.Height div Volume);
      end;
   end;
end;

procedure TFTesteSOM.ShowFFT;
var i, j, k: integer;
   Fator: integer;
begin
   FFT.Size := DATASIZE;

   i := 1;
   while i <= DATASIZE do
   begin
      DataFFT[2*i-1] := (WaveHandle.Buffer[0][i-1] + 32768) / 32768;
      DataFFT[2*i] := 0;
      Inc(i);
   end;

   { Faz FFT }
   FFT.DoFFT(DataFFT);

   { Calcula espectro de frequncias }
   i := 1;
   while (i <= FFT.Size) do
   begin
      Espectro[i] := Sqrt(DataFFT[2*i-1]*DataFFT[2*i-1] + DataFFT[2*i]*DataFFT[2*i]);
      Inc(i);
   end;

   { Limpa TImage }
   ARect := Rect(0, 0, ImageFFT.Width, ImageFFT.Height);
   ImageFFT.Canvas.Brush.Color := clBlack;
   ImageFFT.Canvas.FillRect(ARect);

   { Preenche com espectro }
   ImageFFT.Canvas.Pen.Color := clWhite;
   i := LowCut;
   while (i < BinCount) do
   begin
      ImageFFT.Canvas.MoveTo((i-1)*BinWidth, ImageFFT.Height);
      ImageFFT.Canvas.LineTo((i-1)*BinWidth, ImageFFT.Height  - BinHeight* Trunc(Espectro[i] * ImageFFT.Height/DATASIZE));
      Inc(i);
   end;

   { Preenche com espectro }
   ImageFFT.Canvas.Pen.Color := clWhite;
   i := LowCut;
   ImageFFT.Canvas.MoveTo((i-1)*BinWidth, ImageFFT.Height);
   while (i < BinCount) do
   begin
      ImageFFT.Canvas.LineTo((i-1)*BinWidth, ImageFFT.Height  - BinHeight * Trunc(Espectro[i] * ImageFFT.Height/DATASIZE));
      ImageFFT.Canvas.MoveTo((i-1)*BinWidth, ImageFFT.Height  - BinHeight * Trunc(Espectro[i] * ImageFFT.Height/DATASIZE));
      Inc(i);
   end;

end;

procedure TFTesteSOM.ShowFFTWin;
var I, J, K, N: integer;
   Total: integer;
   F: TextFile;
   FName, FN: string;
   SampleLine: String;
   SampleBin: integer;
   W, WT, S: longint;
   First: boolean;
   V: array[0..9] of integer;
begin
   StatusBar.Panels[0].Text := 'Gerando arquivo de amostra de FFT...';
   LabelDigit.Caption := '';

   FN := Copy(ExtractFileName(OpenDialog.FileName),1,Pos('.',ExtractFileName(OpenDialog.FileName))-1);
   FName := 'SMP' + FN + '.TXT';
   AssignFile(F, FName);
   Rewrite(F);

   FFT.Size := DATASIZE;
   Total := WaveHandle.BytesRemain + WaveHandle.NumBuffers * 32768;

   k := 0;
   WT := 100;
   MemoTest.Lines.Clear;
   while k < Total do
   begin
      i := 1;
      while i <= FFT.Size do
      begin
         DataFFT[2*i-1] := (WaveHandle.Buffer[0][i+k-1] + 32768) / 32768;
         DataFFT[2*i] := 0;
         Inc(i);
      end;

      { Faz FFT }
      FFT.DoFFT(DataFFT);

      { Calcula espectro de frequncias }
      i := 1;
      while (i <= FFT.Size) do
      begin
         Espectro[i] := Sqrt(DataFFT[2*i-1]*DataFFT[2*i-1] + DataFFT[2*i]*DataFFT[2*i]);
         Inc(i);
      end;

      { Limpa TImage }
      ARect := Rect(0, 0, ImageFFT.Width, ImageFFT.Height);
      ImageFFT.Canvas.Brush.Color := clBlack;
      ImageFFT.Canvas.FillRect(ARect);

      { Preenche com espectro }
      SampleLine := '';
      i := LowCut;
      J := 0;
      while (i < BinCount) do
      begin
         if Espectro[i] > 10 then
         begin
            ImageFFT.Canvas.Pen.Color := clRed;
            ImageFFT.Canvas.Brush.Color := clRed;
         end
         else
         if Espectro[i] < 4 then
         begin
            ImageFFT.Canvas.Pen.Color := clBlue;
            ImageFFT.Canvas.Brush.Color := clBlue;
         end
         else
         begin
            ImageFFT.Canvas.Pen.Color := clYellow;
            ImageFFT.Canvas.Brush.Color := clYellow;
         end;

         ImageFFT.Canvas.Rectangle((i-1)*BinWidth-DeltaBin,
                                 ImageFFT.Height,
                                 (i-1)*BinWidth+DeltaBin,
                                 ImageFFT.Height  - BinHeight*Trunc(Espectro[i]*ImageFFT.Height/DATASIZE));

         SampleBin := Trunc(Espectro[i]);
         if SampleBin <= NoiseCut then SampleBin := 0;
         SampleLine := SampleLine + Format('%5d',[SampleBin]);
         X[j] := SampleBin;
         Inc(i);
         Inc(j);
      end;
      SampleLine := SampleLine + Format('%5d',[StrToIntDef(FN,0)]);

      { Faz o somatrio dos bins }
      S := 0;
      for J := 0 to SOMGer.Dimension-1 do
      begin
         S := S + Trunc(X[J]);
      end;

      { Verifica o peso a que pertence a amostra }
      W := SOMGer.SeekWeight(X);

      if (S = 0) and First then
      begin
         {  outro dgito }
         { Escolhe o que ocorreu mais vezes }
         N := 0;
         for J := 1 to 9 do
         begin
            if V[J] > V[N] then
               N := J;
         end;

         { Mostra o dgito escolhido }
         LabelDigit.Caption := LabelDigit.Caption + IntToStr(N);

         { Zera o vetor para o prximo }
         for J := 0 to 9 do
         begin
            V[J] := 0;
         end;
         First := False;
      end
      else
      begin
         { Aumenta uma ocorrncia }
         MemoTest.Lines.Add(StringOfChar(' ',W*3) + IntToStr(W));
         Inc(V[W]);
         First := True;
      end;

      ImageFFT.Canvas.Pen.Color := clWhite;
      i := LowCut;
      while (i < BinCount) do
      begin
         ImageFFT.Canvas.MoveTo((i-1)*BinWidth, ImageFFT.Height);
         ImageFFT.Canvas.LineTo((i-1)*BinWidth, ImageFFT.Height-10);
         Inc(i);
      end;

      WriteLn(F, SampleLine);

      Application.ProcessMessages;
      Sleep(33);
      Inc(k,FFT.Size);
   end;
   CloseFile(F);

   StatusBar.Panels[0].Text := 'Arquivo de amostra de FFT gerado';
end;


procedure TFTesteSOM.BtnTestClick(Sender: TObject);
begin
   { Gera a amostra para o dgito considerado }
   ShowFFTWin;
end;

procedure TFTesteSOM.BtnPlayClick(Sender: TObject);
begin
   { Reproduz o som do dgito }
   WaveHandle.Play;
end;

procedure TFTesteSOM.FormCreate(Sender: TObject);
var I: integer;
begin
   MemoTest.Lines.Clear;

   { Ajusta parmetros iniciais das redes SOM }
   SOMGer.Dimension := 30;
   SOMGer.Points := 110;
   SOMGer.Weights := 10;
   SOMGer.Knowledge := FSoundRec.Conhecimento;

   for I := 0 to SOMGer.Dimension-1 do
   begin
      Minimo[i] := 0;
      Maximo[i] := 2000;
   end;

   SOMGer.Bounds(Minimo, Maximo);
   SOMGer.Build;
   { Carrega arquivo de treinamento }
   SOMGer.Load;
   { Aloca espao para vetor de teste }
   SetLength(X, SOMGer.Dimension);

   LabelDigit.Caption := '';
end;

end.

