unit UTreinamentoSOM;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls, Som;

type
  TFTreinamentoSOM = class(TForm)
    StatusBar: TStatusBar;
    Panel1: TPanel;
    Panel2: TPanel;
    SOMInd: TSOM;
    BtnTrain: TButton;
    MemoTrain: TMemo;
    SOMGer: TSOM;
    procedure BtnTrainClick(Sender: TObject);
    procedure SOMIndIteration(Sender: TObject; Epochs: Integer);
    procedure SOMIndUpdateSOMGauss(Sender: TObject; Epochs: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    { Vetores para manipulao da rede SOM }
    VetorGerSOM, VetorIndSOM: TSOMArrayInput;
    Minimo, Maximo: array[0..30] of extended;
    
  end;

var
  FTreinamentoSOM: TFTreinamentoSOM;

implementation

{$R *.dfm}

uses USoundRec;

procedure TFTreinamentoSOM.BtnTrainClick(Sender: TObject);
var FName, Temp, Head: string;
    FValue: integer;
    i, j, k, m, Match, t: integer;
    F: TextFile;
    P: array[0..61] of extended;
begin
   { FASE 1 - Treinamento Individual }

   { Ajusta parmetros iniciais das redes SOM }
   SOMInd.Dimension := 30; // BinCount - LowCount + 1
   SOMInd.Points := 11;
   SOMInd.Weights := 1;

   SOMInd.RateSOM := FSoundRec.TaxaSOM;
   SOMInd.RateLVQ := FSoundRec.TaxaLVQ;

   SOMGer.Dimension := 30;
   SOMGer.Points := 110;
   SOMGer.Weights := 10;

   SOMGer.Knowledge := FSoundRec.Conhecimento;

   { Aloca vetor para pontos SOM }
   SetLength(VetorIndSOM, SOMInd.Points, SOMInd.Dimension);
   SetLength(VetorGerSOM, SOMGer.Points, SOMGer.Dimension);

   for i := 0 to SOMInd.Dimension-1 do
   begin
      Minimo[i] := 0;
      Maximo[i] := 2000;
   end;

   SOMInd.Bounds(Minimo, Maximo);
   SOMGer.Bounds(Minimo, Maximo);

   SOMInd.Build;
   SOMGer.Build;

   { Faz para todos os dgitos }
   MemoTrain.Lines.Add('Valor dos pesos nas redes SOM individuais');
   MemoTrain.Lines.Add('');

   { Monta cabealho }
   Temp := '    ';
   for j := 1 to 30 do
      Temp := Temp + Format('%3d ',[j]);
   MemoTrain.Lines.Add(Temp);
   MemoTrain.Lines.Add('---------------------------------------------------------------------------------------------------------------------------');

   for i := 0 to 9 do
   begin
      FName := 'C:\Livro\Exemplos\10\Amostras\SMP' + IntToStr(i) + '.TXT';

      AssignFile(F, FName);
      Reset(F);

      { Faz a leitura do arquivo e coloca no vetor SOM }
      j := 0;
      while not Eof(F) do
      begin
         ReadLn(F, Temp);
         for k := 0 to SOMInd.Dimension-1 do
         begin
            FValue := StrToInt(Copy(Temp,k*5+1,5));
            VetorIndSOM[j,k] := FValue;
            VetorGerSOM[j+i*SOMInd.Points,k] := FValue;
         end;
         Inc(j);
      end;

      { Executa algoritmo SOM }
      SOMInd.DoSOMGauss(VetorIndSOM);

      { Mostra valor dos neurnios individuais }
      for j := 0 to SOMInd.Weights-1 do
      begin
         Temp := Format('%3d:',[i]);
         m := i;
         for k := 0 to SOMInd.Dimension-1 do
         begin
            if Round(SOMInd.GetValue(j,k)) <> 0 then
               Temp := Temp + Format('%3d ',[Round(SOMInd.GetValue(j,k))])
            else
               Temp := Temp + '    ';
            { Coloca o valor obtido do neurnio na rede final }
            SOMGer.SetValue(m, k, SOMInd.GetValue(j,k));
         end;
         MemoTrain.Lines.Add(Temp);
      end;

      { Fecha arquivo de dados }
      CloseFile(F);
   end;
   MemoTrain.Lines.Add('');

   { Imprime o valor dos pesos }
   MemoTrain.Lines.Add('Valor dos pesos na rede SOM geral');
   MemoTrain.Lines.Add('');

   { Monta cabealho }
   Temp := '    ';
   for j := 1 to 30 do
      Temp := Temp + Format('%3d ',[j]);
   MemoTrain.Lines.Add(Temp);
   MemoTrain.Lines.Add('---------------------------------------------------------------------------------------------------------------------------');

   for j := 0 to SOMGer.Weights-1 do
   begin
      Temp := Format('%3d:',[j]);
      for i := 0 to SOMGer.Dimension-1 do
      begin
         if Round(SOMGer.GetValue(j,i)) <> 0 then
            Temp := Temp + Format('%3d ',[Round(SOMGer.GetValue(j,i))])
         else
            Temp := Temp + '    ';
      end;
      MemoTrain.Lines.Add(Temp);
   end;
   MemoTrain.Lines.Add('');

   { FASE 2 - Teste Geral }

   { Testa a rede SOM }
   MemoTrain.Lines.Add('Teste da rede para os valores das amostras');
   MemoTrain.Lines.Add('');
   AssignFile(F, 'C:\Livro\Exemplos\10\Amostras\SOMTeste.txt');
   Rewrite(F);
   Match := 0;
   m := 0;

   { Monta cabealho }
   Temp := '   ';
   for t := 1 to 30 do
      Temp := Temp + Format('%3d ',[t]);
   MemoTrain.Lines.Add(Temp);
   MemoTrain.Lines.Add('---------------------------------------------------------------------------------------------------------------------------');

   for j := 0 to SOMGer.Points-1 do
   begin
      Temp := '    ';
      for i := 0 to SOMGer.Dimension-1 do
      begin
         { Coloca no vetor para buscar o peso mais prximo }
         P[i] := VetorGerSOM[j,i];
         if VetorGerSOM[j,i] <> 0 then
            Temp := Temp + Format('%3d ',[Round(VetorGerSOM[j,i])])
         else
            Temp := Temp + '    ';
      end;
      Sleep(33);
      if m <> (j div 11) then
      begin
         m := j div 11;
         WriteLn(F);
         WriteLn(F, Format('%65d',[((Match+1)*100) div 11]) + '% ' + Format('(%5d)',[(Match+1)]));
         WriteLn(F);
         MemoTrain.Lines.Add('');
         MemoTrain.Lines.Add(Format('%65d',[((Match+1)*100) div 11]) + '% ' + Format('(%5d)',[(Match+1)]));
         MemoTrain.Lines.Add('');
         Match := 0;

         { Monta cabealho }
         Head := '    ';
         for t := 1 to 30 do
            Head := Head + Format('%3d ',[t]);
         MemoTrain.Lines.Add(Head);
         MemoTrain.Lines.Add('---------------------------------------------------------------------------------------------------------------------------');
      end;
      if m = k then Inc(Match);
      Temp := Temp + Format('%3d ',[m]);
      k := SOMGer.SeekWeight(P);
      Temp := Temp + Format('%3d ',[k]);
      WriteLn(F, Temp);
      MemoTrain.Lines.Add(Temp);
   end;
   if m <> (j div 11) then
   begin
      m := j div 11;
      WriteLn(F);
      WriteLn(F, Format('%65d',[((Match+1)*100) div 11]) + '% ' + Format('(%5d)',[(Match+1)]));
      WriteLn(F);
      MemoTrain.Lines.Add('');
      MemoTrain.Lines.Add(Format('%65d',[((Match+1)*100) div 11]) + '% ' + Format('(%5d)',[(Match+1)]));
      MemoTrain.Lines.Add('');
      Match := 0;
   end;
   CloseFile(F);

   { Salva o arquivo }
   SOMGer.Save;
end;

procedure TFTreinamentoSOM.SOMIndIteration(Sender: TObject; Epochs: Integer);
begin
    StatusBar.Panels[0].Text := 'pocas: ' + IntToStr(Epochs);
    StatusBar.Repaint;
end;

procedure TFTreinamentoSOM.SOMIndUpdateSOMGauss(Sender: TObject;
  Epochs: Integer);
begin
   with SOMInd do
   begin
      Sigma := 0.5 * Exp(-4*Epochs/Iterations);
      RateSOM := 0.5 * Exp(-0.5*Epochs/Iterations);
      StatusBar.Panels[1].Text := Format('%3d:',[Epochs]) + ' Sigma: ' + Format('%2.4f',[Sigma]) + ' RateSOM: ' + Format('%2.4f',[RateSOM]);
      StatusBar.Repaint;
   end;
end;

procedure TFTreinamentoSOM.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
   Action := caFree;
   FTreinamentoSOM := nil;
end;

end.
