unit UTrein;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, StdCtrls, Buttons, ComCtrls, MLP;

const
  ENTRADAS = 48; // Nmero de amostras iniciais
  PERIODOS = 10; // Repeties do perodo de amostras (1 ciclo)

type
  TTrein = class(TForm)
    tblPrevis: TTable;
    dsPrevis: TDataSource;
    btnConstruir: TBitBtn;
    btnTreinar: TBitBtn;
    StatusBar1: TStatusBar;
    tblPrevisID: TIntegerField;
    tblPrevisHora: TFloatField;
    tblPrevisConsultas: TIntegerField;
    MLP: TMLP;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnConstruirClick(Sender: TObject);
    procedure btnTreinarClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Amostras: array[1..ENTRADAS*PERIODOS] of single;
  end;

var
  Trein: TTrein;

implementation

uses UPrevis;

{$R *.DFM}

procedure TTrein.FormCreate(Sender: TObject);
begin
   tblPrevis.Open;
end;

procedure TTrein.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   tblPrevis.Close;
   Action := caFree;
   Trein := nil;
end;

procedure TTrein.btnConstruirClick(Sender: TObject);
var i, j: integer;
begin
   { Inserir nmero de neurnios e camadas }
   MLP.Struct.Clear;
   MLP.Struct.Add(IntToStr(ENTRADAS)); // camada de entrada
   if FPrevis.NeuroniosOcultos > 0 then
      MLP.Struct.Add(IntToStr(FPrevis.NeuroniosOcultos)); // camada oculta
   MLP.Struct.Add('1'); // camada de sada

   MLP.Build;
   MLP.Learning := FPrevis.Aprendizagem;
   MLP.Momentum := FPrevis.Inercia;
   MLP.Knowledge := FPrevis.Conhecimento;

   { Definio dos valores mximo e mnimo do conjunto de treinamento }
   FPrevis.Minimo := 1E9;
   FPrevis.Maximo := -1E9;
   tblPrevis.First;
   while not tblPrevis.Eof do
   begin
      if tblPrevis['Consultas'] > FPrevis.Maximo then
         FPrevis.Maximo := tblPrevis['Consultas'];
      if tblPrevis['Consultas'] < FPrevis.Minimo then
         FPrevis.Minimo := tblPrevis['Consultas'];
      tblPrevis.Next;
   end;

   { Definio da faixa de trabalho dos neurnios de entrada }
   with MLP do
   begin
      { Consultas }
      for i := 1 to ENTRADAS do
      begin
         SetInputMin(i, FPrevis.Minimo);
         SetInputMax(i, FPrevis.Maximo);
      end;

      { Definio da faixa de trabalho do neurnio de sada }
      SetOutputMin(1, FPrevis.Minimo);
      SetOutputMax(1, FPrevis.Maximo);
   end;

   { Preenche o vetor de amostras }
   j := 1;
   for i := 1 to PERIODOS do
   begin
      tblPrevis.First;
      while not tblPrevis.Eof do
      begin
         Amostras[j] := tblPrevis['Consultas'];
         Inc(j);
         tblPrevis.Next;
      end;
   end;
   
end;

procedure TTrein.btnTreinarClick(Sender: TObject);
var i, k, j: word;
  m: longint;
begin
   { Treinamento }
   m := 0; // conta o nmero de pocas

   { Faz para o nmero de ciclos }
   for j := 0 to FPrevis.Ciclos-1 do
   begin
      { O numero de entradas pelos perodos, deslocando-se uma amostra  frente a  
        cada treinamento }
      for k := 1 to (PERIODOS*ENTRADAS)-ENTRADAS-1 do
      begin
         i := 1;
         while i <= ENTRADAS do
         begin
            MLP.SetInput(i, Amostras[k+i-1]);
            Inc(i);
         end;

         MLP.SetOutput(1, Amostras[k+i-1]);

         MLP.Training;
         Inc(m);
      end;
      StatusBar1.Panels[0].Text := 'pocas: ' + IntToStr(m);
      StatusBar1.Panels[1].Text := Format('Erro Total: %2.6f',[MLP.Cost]);
      StatusBar1.Repaint;
      Application.ProcessMessages;
   end;

   { Aps treinar guarda o conhecimento acumulado pela rede }
   MLP.Save;
end;

end.
