unit UnitSOMGrid2D;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, Math, Som, ComCtrls;

type
  TFSOM = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    btnSOM: TBitBtn;
    Panel3: TPanel;
    StaticText1: TStaticText;
    Image: TImage;
    btnInicial: TBitBtn;
    btnLVQ: TBitBtn;
    Label4: TLabel;
    Label5: TLabel;
    btnSave: TBitBtn;
    btnLoad: TBitBtn;
    SOM: TSOM;
    Label8: TLabel;
    Label9: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    procedure FormActivate(Sender: TObject);
    procedure btnSOMClick(Sender: TObject);
    procedure SOMIteration(Sender: TObject; Epochs: Integer);
    procedure SOMUpdateSOMDiscr(Sender: TObject; Epochs: Integer);
    procedure SOMUpdateLVQ(Sender: TObject; Epochs: Integer);
    procedure btnInicialClick(Sender: TObject);
    procedure btnLVQClick(Sender: TObject);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Bitmap1: TBitmap;
    x, y : TSOMArrayInput;
    Minimo, Maximo: array[0..1] of extended;
  end;

var
  FSOM: TFSOM;

implementation

{$R *.DFM}

procedure TFSOM.FormActivate(Sender: TObject);
begin
   { Ajusta bitmap e constri a rede na primeira vez }
   Bitmap1 := TBitmap.Create;
   Bitmap1.Height := 255;
   Bitmap1.Width := 255;
   SOM.Build;
end;

procedure TFSOM.btnSOMClick(Sender: TObject);
begin
   { Perfaz o treinamento da rede SOM }
   with SOM do DoSOMDiscr(x);
   btnInicial.Enabled := True;
   btnSOM.Enabled := False;
end;

procedure TFSOM.SOMUpdateSOMDiscr(Sender: TObject; Epochs: Integer);
begin
   { Ajuste da taxa de aprendizado e proximidade de vizinhana }
   with SOM do
   begin
      RateSOM := 0.9 * Exp(-6*Epochs/Iterations);
      Lambda := Round(5 * Exp(-3*Epochs/Iterations));
   end;
end;

procedure TFSOM.SOMIteration(Sender: TObject; Epochs: Integer);
var i, j, k1, k2 : integer;
   ARect : TRect;
begin
   { Limpa bitmap  }
   ARect := Rect(0, 0, 255, 255);
   Bitmap1.Canvas.Brush.Color := clWhite;
   Bitmap1.Canvas.FillRect(ARect);

   { Mostra pontos da distribuio }
   Bitmap1.Canvas.Pen.Color := clYellow;
   for j := 0 to SOM.Points-1 do
      Bitmap1.Canvas.Rectangle(Round(x[j,0]), Round(x[j,1]), Round(x[j,0]+5), Round(x[j,1]+5));

   { Mostra pesos }
   Bitmap1.Canvas.Pen.Color := clNavy;
   with SOM do
   begin
      for i := 0 to SOM.Weights-1 do
         Bitmap1.Canvas.Rectangle(Round(GetValue(i,0)), Round(GetValue(i,1)), Round(GetValue(i,0)+5), Round(GetValue(i,1)+5));
   end;

   { Conecta os pesos vizinhos }
   for j := 0 to SOM.AxisX-1 do
   begin
      for i := 0 to SOM.AxisY-2 do
      begin
         k1 := j*SOM.AxisX + i;
         k2 := j*SOM.AxisX + (i+1);
         Bitmap1.Canvas.MoveTo(Round(SOM.GetValue(k1,0)), Round(SOM.GetValue(k1,1)));
         Bitmap1.Canvas.LineTo(Round(SOM.GetValue(k2,0)), Round(SOM.GetValue(k2,1)));
      end;
   end;
   for j := 0 to SOM.AxisX-1 do
   begin
      for i := 0 to SOM.AxisY-2 do
      begin
         k1 := i*SOM.AxisY + j;
         k2 := (i+1)*SOM.AxisY + j;
         Bitmap1.Canvas.MoveTo(Round(SOM.GetValue(k1,0)), Round(SOM.GetValue(k1,1)));
         Bitmap1.Canvas.LineTo(Round(SOM.GetValue(k2,0)), Round(SOM.GetValue(k2,1)));
      end;
   end;

   { Mostra bitmap }
   Image.Picture.Assign(Bitmap1);
   Label8.Caption := IntToStr(Epochs);
   Application.ProcessMessages;
end;

procedure TFSOM.btnInicialClick(Sender: TObject);
var i, j, m, n : integer;
   ARect : TRect;
begin
   { Libera a rede existente }
   SOM.Clear;

   { Define limites }
   for i := 0 to SOM.Dimension-1 do
   begin
      Minimo[i] := 10;
      Maximo[i] := 245;
   end;
   SOM.Bounds(Minimo, Maximo);

   { Constri a rede }
   SOM.Build;

   { Limpa Bitmap }
   ARect := Rect(0, 0, 255, 255);
   Bitmap1.Canvas.Brush.Color := clWhite;
   Bitmap1.Canvas.FillRect(ARect);

   { Aloca vetor de amostras }
   SetLength(x, SOM.Points, SOM.Dimension);

   { Ajusta valores aleatrios para os pesos }
   Bitmap1.Canvas.Pen.Color := clNavy;
   for i := 0 to SOM.Weights-1 do
   begin
      for j := 0 to SOM.Dimension-1 do
         SOM.SetValue(i, j, Random(251));
      Bitmap1.Canvas.Rectangle(Round(SOM.GetValue(i,0)), Round(SOM.GetValue(i,1)), Round(SOM.GetValue(i,0)+5), Round(SOM.GetValue(i,1)+5));
   end;

   { Alimenta valores de entrada aleatrios }
   Bitmap1.Canvas.Pen.Color := clYellow;
   for j := 0 to SOM.Points-1 do
   begin
      x[j,0] := Random(256);
      x[j,1] := Random(256);
      Bitmap1.Canvas.Rectangle(Round(x[j,0]), Round(x[j,1]), Round(x[j,0]+5), Round(x[j,1]+5));
   end;

   { Mostra inicializao }
   Image.Picture.Assign(Bitmap1);
   btnSOM.Enabled := True;
end;

procedure TFSOM.btnLVQClick(Sender: TObject);
var P: array[0..1] of extended;
    i, j : integer;
begin
   { Perfaz aprendizado de quantizao vetorial }
   SOM.DoLVQ4(x);
   btnInicial.Enabled := True;
   btnSOM.Enabled := False;
end;

procedure TFSOM.SOMUpdateLVQ(Sender: TObject; Epochs: Integer);
begin
   { Ajuste da taxa de aprendizado de quantizao vetorial }
   with SOM do
      RateLVQ := 0.01 * Exp(-6*Epochs/Iterations);
end;

procedure TFSOM.ImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var P: array[0..1] of extended;
   i : integer;
begin
   { Mostra peso mais prximo do ponto clicado }
   P[0] := X;
   P[1] := Y;
   i := SOM.SeekWeight(P);
   Label5.Caption := FloatToStr(i) + '/ X = ' + Format('%3.2f',[SOM.GetValue(i,0)])  + ' / Y = ' + Format('%3.2f',[SOM.GetValue(i,1)]);
end;

procedure TFSOM.btnSaveClick(Sender: TObject);
begin
   { Salva a rede }
   SOM.Save;
end;

procedure TFSOM.btnLoadClick(Sender: TObject);
begin
   { Carrega a rede salva anteriormente }
   SOM.Load;
   SOMIteration(Sender, 0);
end;

end.
