unit WaveHandle;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem;

type
  TMPDeviceTypes = (dtAutoSelect, dtAVIVideo, dtCDAudio, dtDAT, dtDigitalVideo, dtMMMovie,
    dtOther, dtOverlay, dtScanner, dtSequencer, dtVCR, dtVideodisc, dtWaveAudio);

  TWaveRIFFHeader = record
     ID : array[0..3] of Char;
     Len: longint;
     IDWave: array[0..3] of Char;
  end;

  TWaveFormatChunk = record
     ID: array[0..3] of Char;
     Len: longint;
     FormatTag: word;
     Channels: word;
     SamplesPerSec: longint;
     AvgBytesPerSec: longint;
     BlockAlign: word;
     BitsPerSample: word;
  end;

  TWaveDataChunk = record
     ID: array[0..3] of Char;
     Len: longint;
  end;

  TWaveFactChunk = record
     ID: array[0..3] of Char;
     SampleLength: longint;
     Len: longint;
  end;

  TArrayWave = array[0..32767] of SmallInt;

  TWaveHandle = class(TComponent)
  private
    { Private declarations }
    FWAVFile: string;
    FLength: longint;
    FLengthFormat: longint;
    FFormatTag: word;
    FChannels: word;
    FSamplesPerSec: longint;
    FAvgBytesPerSec: longint;
    FBlockAlign: word;
    FBitsPerSample: word;
    FLengthData: longint;
    FNumBuffers: longint;
    FBytesRemain: longint;
    procedure SetWAVFile(Value: string);
  protected
    { Protected declarations }
    F, n, r: integer;
  public
    { Public declarations }
    WaveRIFFHeader: TWaveRIFFHeader;
    WaveFormatChunk: TWaveFormatChunk;
    WaveDataChunk: TWaveDataChunk;
    WaveFactChunk: TWaveFactChunk;
    Buffer: array of TArrayWave;
    constructor Create(AOwner: TComponent); override;
    procedure Load;
    procedure Save;
    procedure Clear;
    procedure Play;
    procedure LoadData(FileName: string; var DataStream: array of SmallInt);
    procedure SaveData(FileName: string; SizeData: integer; var DataStream: array of SmallInt);
  published
    { Published declarations }
    property WAVFile: string read FWAVFile write SetWAVFile;
    property Length: longint read FLength;
    property LengthFormat: longint read FLengthFormat;
    property FormatTag: word read FFormatTag;
    property Channels: word read FChannels;
    property SamplesPerSec: longint read FSamplesPerSec;
    property AvgBytesPerSec: longint read FAvgBytesPerSec;
    property BlockAlign: word read FBlockAlign;
    property BitsPerSample: word read FBitsPerSample;
    property LengthData: longint read FLengthData;
    property NumBuffers: longint read FNumBuffers; // n de buffers de 32768 necessrios
    property BytesRemain: longint read FBytesRemain; // bytes restantes
  end;

procedure Register;

implementation

procedure TWaveHandle.LoadData(FileName: string; var DataStream: array of SmallInt);
var i, j, k : longint;
begin
   F := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
   if F > 0 then
   begin
      FileRead(F, WaveRIFFHeader, SizeOf(WaveRIFFHeader));
      FileRead(F, WaveFormatChunk, SizeOf(WaveFormatChunk));
      FileRead(F, WaveFactChunk, SizeOf(WaveFactChunk));
      if WaveFactChunk.ID = 'data' then
         FileSeek(F, -SizeOf(WaveFactChunk), 1);
      FileRead(F, WaveDataChunk, SizeOf(WaveDataChunk));
      n := (WaveDataChunk.Len div 2) div 32768; // n de buffers necessrios
      r := (WaveDataChunk.Len div 2) mod 32768; // bytes restantes
      { Aloca buffers para leitura do arquivo }
      SetLength(Buffer, n+1); // o resto ficar em n+1

      i := 0;
      while i < n do
      begin
         FileRead(F, Buffer[i], SizeOf(Buffer[i]));
         Inc(i);
      end;
      FileRead(F, Buffer[n], r*2);

      // Coloca os dados enfileriados em DataStream[]
      k := 0;
      i := 0;
      while i < n do
      begin
         for j := 0 to 32767 do
         begin
            DataStream[k] := Buffer[i][j];
            Inc(k);
         end;
         Inc(i);
      end;

      for j := 0 to r do
      begin
         DataStream[k] := Buffer[n][j];
         Inc(k);
      end;

      FileClose(F);
   end;
end;

procedure TWaveHandle.Load;
var i, j, k : longint;
begin
   F := FileOpen(FWAVFile, fmOpenRead or fmShareDenyNone);
   if F > 0 then
   begin
      FileRead(F, WaveRIFFHeader, SizeOf(WaveRIFFHeader));
      FileRead(F, WaveFormatChunk, SizeOf(WaveFormatChunk));
      FileRead(F, WaveFactChunk, SizeOf(WaveFactChunk));
      if WaveFactChunk.ID = 'data' then
         FileSeek(F, -SizeOf(WaveFactChunk), 1);
      FileRead(F, WaveDataChunk, SizeOf(WaveDataChunk));
      n := (WaveDataChunk.Len div 2) div 32768; // n de buffers necessrios
      r := (WaveDataChunk.Len div 2) mod 32768; // bytes restantes

      SetLength(Buffer, n+1); // o resto ficar no prximo

      i := 0;
      while i < n do
      begin
         FileRead(F, Buffer[i], SizeOf(Buffer[i]));
         Inc(i);
      end;
      FileRead(F, Buffer[n], r*2);

      FileClose(F);

      // Coloca os valores nas propriedades
      FLength := WaveRIFFHeader.Len;
      FLengthFormat := WaveFormatChunk.Len;
      FFormatTag := WaveFormatChunk.FormatTag;
      FChannels := WaveFormatChunk.Channels;
      FSamplesPerSec := WaveFormatChunk.SamplesPerSec;
      FAvgBytesPerSec := WaveFormatChunk.AvgBytesPerSec;
      FBlockAlign := WaveFormatChunk.BlockAlign;
      FBitsPerSample := WaveFormatChunk.BitsPerSample;
      FLengthData := WaveDataChunk.Len;
      FNumBuffers := n;
      FBytesRemain := r;
   end;
end;

procedure TWaveHandle.Save;
var i, j, k: longint;
begin
   F := FileCreate(FWAVFile);
   if F > 0 then
   begin
      FileWrite(F, WaveRIFFHeader, SizeOf(WaveRIFFHeader));
      FileWrite(F, WaveFormatChunk, SizeOf(WaveFormatChunk));
      FileWrite(F, WaveDataChunk, SizeOf(WaveDataChunk));

      i := 0;
      while i < n do
      begin
         FileWrite(F, Buffer[i], SizeOf(Buffer[i]));
         Inc(i);
      end;
      FileWrite(F, Buffer[n], r*2);
      FileClose(F);
   end;
end;

procedure TWaveHandle.SaveData(FileName: string; SizeData: integer; var DataStream: array of SmallInt);
var i, j, k: longint;
begin
   F := FileCreate(FileName);
   if F > 0 then
   begin
      FileWrite(F, WaveRIFFHeader, SizeOf(WaveRIFFHeader));
      FileWrite(F, WaveFormatChunk, SizeOf(WaveFormatChunk));
      FileWrite(F, WaveDataChunk, SizeOf(WaveDataChunk));
      FileWrite(F, DataStream, SizeData);
      FileClose(F);
   end;
end;

procedure TWaveHandle.Clear;
begin
   SetLength(Buffer, 0); // o resto ficar no prximo
   FWAVFile := '';
   FLength := 0;
   FLengthFormat := 0;
   FFormatTag := 0;
   FChannels := 0;
   FSamplesPerSec := 0;
   FAvgBytesPerSec := 0;
   FBlockAlign := 0;
   FBitsPerSample := 0;
   FLengthData := 0;
   FNumBuffers := 0;
   FBytesRemain := 0;
end;

procedure TWaveHandle.Play;
const
  DeviceName: array[0..12] of PChar = ('', 'AVIVideo', 'CDAudio', 'DAT',
    'DigitalVideo', 'MMMovie', 'Other', 'Overlay', 'Scanner', 'Sequencer',
    'VCR', 'Videodisc', 'WaveAudio');

var PlayParm: TMCI_Play_Parms;
    OpenParm: TMCI_Open_Parms;
    GenParm: TMCI_Generic_Parms;
    FError: longint;
    FFlags : longint;
    FDeviceID: word;
    FElement : string;
    FErrorText: array[0..127] of Char;
    FErrorString: string;
begin
   OpenParm.dwCallback := 0;
   OpenParm.lpstrDeviceType := 'WaveAudio'; // WaveAudio
   FElement := FWAVFile;
   OpenParm.lpstrElementName := PChar(FElement);
   OpenParm.lpstrAlias := PChar(0);

   FFlags := mci_Notify;
   FFlags := FFlags or mci_Open_Element or mci_Open_Type;
   OpenParm.dwCallback := 0;
   FError := mciSendCommand(0, mci_Open, FFlags, Longint(@OpenParm));

   mciGetErrorString(FError, FErrorText, 128);
   FErrorString := StrPas(FErrorText);

   if FError = 0 then {problem opening device}
   begin
      FDeviceID := OpenParm.wDeviceID;
      FFlags := mci_Wait;
      PlayParm.dwCallback := 0;
      FError := mciSendCommand( FDeviceID, mci_Play, FFlags, Longint(@PlayParm));

      mciGetErrorString(FError, FErrorText, 128);
      FErrorString := StrPas(FErrorText);

      if FError <> 0 then
         raise Exception.Create(FErrorString);

      GenParm.dwCallback := 0;
      FError := mciSendCommand( FDeviceID, mci_Close, FFlags, Longint(@GenParm));

      mciGetErrorString(FError, FErrorText, 128);
      FErrorString := StrPas(FErrorText);

      if FError <> 0 then
         raise Exception.Create(FErrorString);
   end
   else
      raise Exception.Create(FErrorString);
end;

procedure TWaveHandle.SetWAVFile(Value: string);
begin
   FWAVFile := Value;
end;

constructor TWaveHandle.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FWAVFile := '';
   FLength := 0;
   FLengthFormat := 0;
   FFormatTag := 0;
   FChannels := 0;
   FSamplesPerSec := 0;
   FAvgBytesPerSec := 0;
   FBlockAlign := 0;
   FLengthData := 0;
   FNumBuffers := 0;
   FBytesRemain := 0;
end;

procedure Register;
begin
  RegisterComponents('Lfm', [TWaveHandle]);
end;

end.
