unit Som;

interface

uses Classes, SysUtils, Dialogs;

type
  PWeight = ^TWeight;

  TWeight = record
     Number : integer; // identificao do peso
     Value : array of extended; // valores assumidos pelo peso nas dimensoes
     X, Y : integer; // para o caso de rodar DoSOMDiscr
     Next: pointer; // ponteiro para proximo peso
  end;

  TMappingEvent = procedure (Sender : TObject; Epochs: longint) of object;

  TSOMArrayInput = array of array of extended;

  TSOMType = (st1D,st2D);

  TSOM = class(TComponent)
  private
     FDimension: word;
     FWeights: integer;
     FPoints: integer;
     FLambda: integer;
     FSigma: double;
     FRateSOM: double;
     FRateLVQ: double;
     FIterations: longint;
     FAxisX: integer;
     FAxisY: integer;
     FSOMType: TSOMType;
     FKnowledge : string; // nome do arquivo de pesos
     FOnUpdateSOMDiscr: TMappingEvent;
     FOnIteration: TMappingEvent;
     FOnUpdateSOMGauss: TMappingEvent;
     FOnUpdateLVQ: TMappingEvent;
     procedure SetDimension(Value: word);
     procedure SetWeights(Value: integer);
     procedure SetPoints(Value: integer);
     procedure SetLambda(Value: integer);
     procedure SetSigma(Value: double);
     procedure SetRateSOM(Value: double);
     procedure SetIterations(Value: longint);
     procedure SetRateLVQ(Value: double);
     procedure SetAxisX(Value: integer);
     procedure SetAxisY(Value: integer);
     procedure SetKnowledge(Value: string);
     procedure SetSOMType(Value: TSOMType);
  protected
     Prepared : boolean;
     FDelta: double;
     FLower, FUpper: array of extended;
     First: PWeight;
     W, V : PWeight;
     function GetWeight : PWeight;
     procedure FreeWeight(var Node : PWeight);
  public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     procedure Build; // constroi o conjunto de pesos
     procedure DoSOMDiscr(var X: TSOMArrayInput);
     procedure DoSOMGauss(var X: TSOMArrayInput);
     procedure DoLVQS(var X: TSOMArrayInput);
     procedure DoLVQNS(var X: TSOMArrayInput);
     procedure Bounds(var Lower, Upper: array of extended);
     function GetValue(Index: integer; Dimension: word): extended;
     procedure SetValue(Index: integer; Dimension: word; Value: extended);
     procedure Clear;  // Elimina a rede SOM
     function SeekWeight(X: array of extended): longint; // busca peso mais prximo
     procedure Save; // salva pesos em arquivo
     procedure Load; // carrega pesos em arquivo, com rede j construda com BUILD
  published
     property Dimension: word read FDimension write SetDimension;
     property Weights: integer read FWeights write SetWeights;
     property Points: integer read FPoints write SetPoints;
     property Lambda: integer read FLambda write SetLambda;
     property Sigma: double read FSigma write SetSigma;
     property RateSOM: double read FRateSOM write SetRateSOM;
     property RateLVQ: double read FRateLVQ write SetRateLVQ;
     property Iterations: longint read FIterations write SetIterations;
     property AxisX: integer read FAxisX write SetAxisX;
     property AxisY: integer read FAxisY write SetAxisY;
     property SOMType: TSOMType read FSOMType write SetSOMType;
     property Knowledge: string read FKnowledge write SetKnowledge;
     property OnUpdateSOMDiscr: TMappingEvent read FOnUpdateSOMDiscr write FOnUpdateSOMDiscr;
     property OnIteration: TMappingEvent read FOnIteration write FOnIteration;
     property OnUpdateSOMGauss: TMappingEvent read FOnUpdateSOMGauss write FOnUpdateSOMGauss;
     property OnUpdateLVQ: TMappingEvent read FOnUpdateLVQ write FOnUpdateLVQ;
  end;

procedure Register;

implementation

procedure TSOM.SetDimension(Value: word);
begin
   FDimension := Value;
end;

procedure TSOM.SetWeights(Value: integer);
begin
   FWeights := Value;
end;

procedure TSOM.SetPoints(Value: integer);
begin
   FPoints := Value;
end;

procedure TSOM.SetLambda(Value: integer);
begin
   FLambda := Value;
end;

procedure TSOM.SetSigma(Value: double);
begin
   FSigma := Value;
end;

procedure TSOM.SetRateSOM(Value: double);
begin
   FRateSOM := Value;
end;

procedure TSOM.SetRateLVQ(Value: double);
begin
   FRateLVQ := Value;
end;

procedure TSOM.SetIterations(Value: longint);
begin
   FIterations := Value;
end;

procedure TSOM.SetAxisX(Value: integer);
begin
   FAxisX := Value;
end;

procedure TSOM.SetAxisY(Value: integer);
begin
   FAxisY := Value;
end;

procedure TSOM.SetKnowledge(Value: string);
begin
   FKnowledge := Value;
end;

procedure TSOM.SetSOMType(Value: TSOMType);
begin
   FSOMType := Value;
end;

function TSOM.GetWeight : PWeight;
var P : PWeight;
begin
   New(P);
   P.Next := nil;
   Result := P;
end;

procedure TSOM.FreeWeight(var Node : PWeight);
begin
   Dispose(Node);
end;

procedure TSOM.Build;
var i, j : word;
   x, y : integer;
begin
   x := 0;
   y := 0;
   W := nil;
   if not Prepared then
   begin
      for i := 0 to FWeights-1 do
      begin
         V := GetWeight;
         if i = 0 then
         begin
            First := V;
            First.Number := i;
            SetLength(First.Value, FDimension);
            for j := 0 to FDimension-1 do First.Value[j] := Random(1000)/1000;
            First.X := x;
            First.Y := y;
            W := First;
         end
         else
         begin
            W.Next := V;
            W := V;
            W.Number := i;
            SetLength(W.Value, FDimension);
            for j := 0 to FDimension-1 do W.Value[j] := Random(1000)/1000;
            W.X := x;
            W.Y := y;
            W.Next := nil; // o ultimo ja fica com nil
         end;
         Inc(x);
         if x = FAxisX then
         begin
            if FSOMType = st2D then
            begin
               x := 0;
               Inc(y);
            end;
         end;
      end;
   end
   else
      MessageDlg('Rede SOM j construda', mtError, [mbOk], 0);
   Prepared := true;
end;

procedure TSOM.DoSOMDiscr(var X: TSOMArrayInput);
// X - 0..FPoints-1,0..FDimension-1
var W, M : PWeight;
   i : word;
   XN, d, dmin, Soma : extended;
   k : longint;
   j : integer;
begin
   M := GetWeight;
   SetLength(M.Value, FDimension);
   for k := 1 to FIterations do
   begin
      j := Random(FPoints);
      dmin := MaxInt;
      W := First;
      // busca minimo peso
      while W <> nil do
      begin
         Soma := 0;
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
//            Soma := Soma + (X[j,i] - W.Value[i])*(X[j,i] - W.Value[i]);
            Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
         end;

//         d := Round(Sqrt(Soma));
         d := Sqrt(Soma);
         if d < dmin then
         begin
            for i := 0 to FDimension-1 do
               M.Value[i] := W.Value[i];
            M.X := W.X;
            M.Y := W.Y;
            dmin := d;
         end;
         W := W.Next;
      end;

      // evento para atualizar Lambda e Rate
      if Assigned(FOnUpdateSOMDiscr) then FOnUpdateSOMDiscr(self, k);

      // Ajusta peso
      W := First;
      while W <> nil do
      begin
         if FSOMType = st2D then
         begin
            if (Abs(M.X - W.X) <= FLambda) and (Abs(M.Y - W.Y) <= FLambda) then
               for i := 0 to FDimension-1 do
               begin
                  XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
//                  W.Value[i] := W.Value[i] + FRateSOM * (X[j,i] - W.Value[i]);
                  W.Value[i] := W.Value[i] + FRateSOM * (XN - W.Value[i]);
               end;
         end
         else
         if FSOMType = st1D then
         begin
            if (Abs(M.X - W.X) <= FLambda) then
               for i := 0 to FDimension-1 do
               begin
                  XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
//                  W.Value[i] := W.Value[i] + FRateSOM * (X[j,i] - W.Value[i]);
                  W.Value[i] := W.Value[i] + FRateSOM * (XN - W.Value[i]);
               end;
         end;

         W := W.Next;
      end;
      if Assigned(FOnIteration) then FOnIteration(self, k);
   end;
   FreeWeight(M);
end;


procedure TSOM.DoSOMGauss(var X: TSOMArrayInput);
// X - 0..FPoints-1,0..FDimension-1
var W, M : PWeight;
   i : word;
   d, dmin, Soma, XN : extended;
   k : longint;
   j : integer;
begin
   Randomize;
   M := GetWeight;
   SetLength(M.Value, FDimension);
   for k := 1 to FIterations do
   begin
      j := Random(FPoints);
      dmin := MaxInt;
      W := First;
      // busca minimo peso
      while W <> nil do
      begin
         Soma := 0;
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
//            Soma := Soma + (X[j,i] - W.Value[i])*(X[j,i] - W.Value[i]);
            Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
         end;

//         d := Round(Sqrt(Soma));
         d := Sqrt(Soma);
         if d < dmin then
         begin
            for i := 0 to FDimension-1 do
               M.Value[i] := W.Value[i];

            dmin := d;
         end;
         W := W.Next;
      end;

      // evento para atualizar Lambda e Rate
      if Assigned(FOnUpdateSOMGauss) then FOnUpdateSOMGauss(self, k);

      // Ajusta peso
      W := First;
      while W <> nil do
      begin
         Soma := 0;
         for i := 0 to FDimension-1 do
            Soma := Soma + (M.Value[i] - W.Value[i])*(M.Value[i] - W.Value[i]);

//         d := Round(Sqrt(Soma));
         d := Sqrt(Soma);

         FDelta := exp(-d*d/(2*FSigma*FSigma));
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
//            W.Value[i] := W.Value[i] + FRateSOM * FDelta * (X[j,i] - W.Value[i]);
            W.Value[i] := W.Value[i] + FRateSOM * FDelta * (XN - W.Value[i]);
         end;
         
         W := W.Next;
      end;
      if Assigned(FOnIteration) then FOnIteration(self, k);
   end;
   FreeWeight(M);
end;

function TSOM.GetValue(Index: integer; Dimension: word): extended;
var W : PWeight;
begin
   Result := 0;
   W := First;
   while W <> nil do
   begin
      if W.Number = Index then
      begin
//         Result := W.Value[Dimension];
         Result := W.Value[Dimension] * (FUpper[Dimension] - FLower[Dimension]) + FLower[Dimension];;
         break;
      end;
      W := W.Next;
   end;
end;

procedure TSOM.SetValue(Index: integer; Dimension: word; Value: extended);
var W : PWeight;
begin
   W := First;
   while W <> nil do
   begin
      if W.Number = Index then
      begin
//         W.Value[Dimension] := Value;
         W.Value[Dimension] := (Value - FLower[Dimension])/(FUpper[Dimension] - FLower[Dimension]);
         break;
      end;
      W := W.Next;
   end;
end;

procedure TSOM.Clear;  // Elimina a rede SOM
var W, V : PWeight;
begin
   if Prepared then
   begin
      while First.Next <> nil do
      begin
         W := First;
         V := W;
         while W.Next <> nil do
         begin
            V := W;
            W := W.Next;
         end;
         V.Next := nil;
         FreeWeight(W);
      end;
      First := nil;
      Prepared := False;
   end
   else
      MessageDlg('Rede SOM no existe.', mtError, [mbOk], 0);
end;

function TSOM.SeekWeight(X: array of extended): longint;
// X - 0..FDimension-1
var W, M : PWeight;
   i : word;
   Soma, XN : extended;
   d, dmin : extended;
begin
   M := GetWeight;
   SetLength(M.Value, FDimension);
   dmin := MaxInt;
   W := First;
   // busca minimo peso
   while W <> nil do
   begin
      Soma := 0;
      for i := 0 to FDimension-1 do
      begin
         XN := (X[i] - FLower[i])/(FUpper[i] - FLower[i]);
         Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
//         Soma := Soma + (X[i] - W.Value[i])*(X[i] - W.Value[i]);
      end;

      d := Sqrt(Soma);
      if d < dmin then
      begin
         for i := 0 to FDimension-1 do
            M.Value[i] := W.Value[i];
         M.Number := W.Number;
         dmin := d;
      end;
      W := W.Next;
   end;
   Result := M.Number;
end;

procedure TSOM.Save;
var F: TextFile;
    W : PWeight;
    i : integer;
begin
   AssignFile(F, FKnowledge);
   Rewrite(F);
   W := First;
   WriteLn(F, Format('%d',[FWeights]));
   WriteLn(F, Format('%d',[FDimension]));
   while W <> nil do
   begin
      WriteLn(F, Format('%d',[W.Number]));
      WriteLn(F, Format('%d',[W.X]));
      WriteLn(F, Format('%d', [W.Y]));
      for i := 0 to FDimension-1 do
         WriteLn(F, Format('%4.12f',[W.Value[i]]));
      W := W.Next;
   end;
   CloseFile(F);
end;

procedure TSOM.Load;
var F: TextFile;
    W : PWeight;
    i : integer;
    Temp: string;
begin
   AssignFile(F, FKnowledge);
   Reset(F);

   Clear; // destri a rede SOM atual

   ReadLn(F, FWeights);
   ReadLn(F, FDimension);

   Build; // constri novamente com os parmetros modificados

   W := First;
   while W <> nil do
   begin
      ReadLn(F, W.Number);
      ReadLn(F, W.X);
      ReadLn(F, W.Y);
      for i := 0 to FDimension-1 do
      begin
         ReadLn(F, Temp);
         W.Value[i] := StrToFloat(Temp);
      end;
      W := W.Next;
   end;
   CloseFile(F);
end;


procedure TSOM.DoLVQS(var X: TSOMArrayInput);
// X - 0..FPoints-1,0..FDimension (um a mais)
var W, M : PWeight;
   i : word;
   Soma, XN : extended;
   d, dmin: extended;
   k, kmin1, kmin2 : longint;
   j : integer;
begin
   Randomize;
   M := GetWeight;
   SetLength(M.Value, FDimension);
   for k := 1 to FIterations do
   begin
      j := Random(FPoints);
      dmin := MaxInt;
      W := First;
      // busca primeiro peso mais prximo
      while W <> nil do
      begin
         Soma := 0;
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
            Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
         end;

         d := Sqrt(Soma);
         if d < dmin then
         begin
            M.Number := W.Number;
            for i := 0 to FDimension-1 do
               M.Value[i] := W.Value[i];

            dmin := d;
         end;
         W := W.Next;
      end;

      // evento para atualizar Lambda e Rate
      if Assigned(FOnUpdateLVQ) then FOnUpdateLVQ(self, k);

      // Encontra o peso
      W := First;
      while W <> nil do
      begin
         if W.Number = M.Number then
            Break;
         W := W.Next;
      end;

      { Se o neurnio escolhido for igual  classe determinada }
      if W.Number = X[j,FDimension] then
      begin
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
            FDelta := W.Value[i] + FRateLVQ * (XN - W.Value[i]);
            if (FDelta >= 0) and (FDelta <= 1) then
               W.Value[i] := FDelta;
         end;
      end
      else
      begin
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
            FDelta := W.Value[i] - FRateLVQ * (XN - W.Value[i]);
            if (FDelta >= 0) and (FDelta <= 1) then
               W.Value[i] := FDelta;
         end;
      end;

      if Assigned(FOnIteration) then FOnIteration(self, k);
   end;
   FreeWeight(M);
end;

procedure TSOM.DoLVQNS(var X: TSOMArrayInput);
// X - 0..FPoints-1,0..FDimension-1
var W, M : PWeight;
   i : word;
   Soma, XN : extended;
   d, dmin: extended;
   k, kmin1, kmin2 : longint;
   j : integer;
begin
   Randomize;
   M := GetWeight;
   SetLength(M.Value, FDimension);
   for k := 1 to FIterations do
   begin
      j := Random(FPoints);
      dmin := MaxInt;
      kmin1 := 0;
      W := First;
      // busca primeiro peso mais prximo
      while W <> nil do
      begin
         Soma := 0;
         for i := 0 to FDimension-1 do
         begin
            XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
            Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
         end;

         d := Sqrt(Soma);
         if d < dmin then
         begin
            for i := 0 to FDimension-1 do
               M.Value[i] := W.Value[i];

            dmin := d;
            kmin1 := W.Number;
         end;
         W := W.Next;
      end;

      dmin := MaxInt;
      kmin2 := 0;
      W := First;
      // busca segundo peso mais prximo
      while W <> nil do
      begin
         if (W.Number <> kmin1) then
         begin
            Soma := 0;
            for i := 0 to FDimension-1 do
            begin
               XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
               Soma := Soma + (XN - W.Value[i])*(XN - W.Value[i]);
            end;

            d := Sqrt(Soma);
            if d < dmin then
            begin
               for i := 0 to FDimension-1 do
                  M.Value[i] := W.Value[i];

               dmin := d;
               kmin2 := W.Number;
            end;
         end;
         W := W.Next;
      end;

      // evento para atualizar Lambda e Rate
      if Assigned(FOnUpdateLVQ) then FOnUpdateLVQ(self, k);

      // Ajusta peso
      W := First;
      while W <> nil do
      begin
         if W.Number = kmin1 then
            for i := 0 to FDimension-1 do
            begin
               XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
               FDelta := W.Value[i] + FRateLVQ * (XN - W.Value[i]);
               if (FDelta >= 0) and (FDelta <= 1) then
                  W.Value[i] := FDelta;
            end;

         if W.Number = kmin2 then
            for i := 0 to FDimension-1 do
            begin
               XN := (X[j,i] - FLower[i])/(FUpper[i] - FLower[i]);
               FDelta := W.Value[i] - FRateLVQ * (XN - W.Value[i]);
               if (FDelta >= 0) and (FDelta <= 1) then
                  W.Value[i] := FDelta;
            end;

         W := W.Next;
      end;
      if Assigned(FOnIteration) then FOnIteration(self, k);
   end;
   FreeWeight(M);
end;

procedure TSOM.Bounds(var Lower, Upper: array of extended);
var i : integer;
begin
   SetLength(FLower, FDimension);
   SetLength(FUpper, FDimension);
   for i := 0 to FDimension-1 do
   begin
      FLower[i] := Lower[i];
      FUpper[i] := Upper[i];
   end;
end;

constructor TSOM.Create(AOwner: TComponent);
begin
   SetDimension(2);
   SetWeights(100);
   SetPoints(100);
   SetLambda(5);
   SetRateSOM(0.9);
   SetRateLVQ(0.01);
   SetIterations(2000);
   SetAxisX(10);
   SetAxisY(10);
   SetKnowledge('SOMFile1');
   SetSOMType(st2D);
   Prepared := false;
   inherited Create(AOwner);
end;

destructor TSOM.Destroy;
begin
   inherited Destroy;
end;

procedure Register;
begin
   RegisterComponents('Lfm', [TSOM]);
end;

end.
